Option Strict Off
Option Explicit On
Module twain
	
	'
	'ȉ twain.h vȂ̂𔲐
	'
	
	'/* There are four containers used for capabilities negotiation:
	'*    TWON_ONEVALUE, TWON_RANGE, TWON_ENUMERATION, TWON_ARRAY
	'* In each container structure ItemType can be TWTY_INT8, TWTY_INT16, etc.
	'* The kind of data stored in the container can be determined by doing
	'* DCItemSize[ItemType] where the following is defined in TWAIN glue code:
	'*          DCItemSize[]= { sizeof(TW_INT8),
	'*                          sizeof(TW_INT16),
	'*                          etc.
	'*                          sizeof(TW_UINT32) };
	'*
	'*/
	
	Public Const TWTY_INT8 As Short = &H0s '/* Means Item is a TW_INT8   */
	Public Const TWTY_INT16 As Short = &H1s '/* Means Item is a TW_INT16  */
	Public Const TWTY_INT32 As Short = &H2s '/* Means Item is a TW_INT32  */
	
	Public Const TWTY_UINT8 As Short = &H3s '/* Means Item is a TW_UINT8  */
	Public Const TWTY_UINT16 As Short = &H4s '/* Means Item is a TW_UINT16 */
	Public Const TWTY_UINT32 As Short = &H5s '/* Means Item is a TW_UINT32 */
	
	Public Const TWTY_BOOL As Short = &H6s '/* Means Item is a TW_BOOL   */
	
	Public Const TWTY_FIX32 As Short = &H7s '/* Means Item is a TW_FIX32  */
	
	Public Const TWTY_FRAME As Short = &H8s '/* Means Item is a TW_FRAME  */
	
	Public Const TWTY_STR32 As Short = &H9s '/* Means Item is a TW_STR32  */
	Public Const TWTY_STR64 As Short = &HAs '/* Means Item is a TW_STR64  */
	Public Const TWTY_STR128 As Short = &HBs '/* Means Item is a TW_STR128 */
	Public Const TWTY_STR255 As Short = &HCs '/* Means Item is a TW_STR255 */
	Public Const TWTY_STR1024 As Short = &HDs '/* Means Item is a TW_STR1024...added 1.9 */
	Public Const TWTY_UNI512 As Short = &HEs '/* Means Item is a TW_UNI512...added 1.9 */
	
	'/****************************************************************************
	'* Capability Constants                                                     *
	'****************************************************************************/
	
	'/* ICAP_IMAGEFILEFORMAT values (FF_means File Format)   */
	Public Const TWFF_TIFF As Short = 0 '/* Tagged Image File Format     */
	Public Const TWFF_PICT As Short = 1 '/* Macintosh PICT               */
	Public Const TWFF_BMP As Short = 2 '/* Windows Bitmap               */
	Public Const TWFF_XBM As Short = 3 '/* X-Windows Bitmap             */
	Public Const TWFF_JFIF As Short = 4 '/* JPEG File Interchange Format */
	Public Const TWFF_FPX As Short = 5 '/* Flash Pix                    */
	Public Const TWFF_TIFFMULTI As Short = 6 '/* Multi-page tiff file         */
	Public Const TWFF_PNG As Short = 7
	Public Const TWFF_SPIFF As Short = 8
	Public Const TWFF_EXIF As Short = 9
	
	'/****************************************************************************
	'* Language Constants                                                       *
	'****************************************************************************/
	Public Const TWLG_JAPANESE As Short = 68
	Public Const TWCY_JAPAN As Short = 81
	
	'/****************************************************************************
	'* Messages                                                                 *
	'****************************************************************************/
	
	'/* All message constants are unique.
	'* Messages are grouped according to which DATs they are used with.*/
	
	Public Const MSG_NULL As Short = &H0s '/* Used in TW_EVENT structure               */
	Public Const MSG_CUSTOMBASE As Short = &H8000s '/* Base of custom messages                  */
	
	'/* Generic messages may be used with any of several DATs.                   */
	Public Const MSG_GET As Short = &H1s '/* Get one or more values                   */
	Public Const MSG_GETCURRENT As Short = &H2s '/* Get current value                        */
	Public Const MSG_GETDEFAULT As Short = &H3s '/* Get default (e.g. power up) value        */
	Public Const MSG_GETFIRST As Short = &H4s '/* Get first of a series of items, e.g. DSs */
	Public Const MSG_GETNEXT As Short = &H5s '/* Iterate through a series of items.       */
	Public Const MSG_SET As Short = &H6s '/* Set one or more values                   */
	Public Const MSG_RESET As Short = &H7s '/* Set current value to default value       */
	Public Const MSG_QUERYSUPPORT As Short = &H8s '/* Get supported operations on the cap.     */
	
	'****************************************************************************
	'* TWAIN Version                                                            *
	'****************************************************************************
	Public Const TWON_PROTOCOLMINOR As Short = 9 ' /* Changed for Version 1.9   */
	Public Const TWON_PROTOCOLMAJOR As Short = 1
	'****************************************************************************
	'* Data Groups                                                              *
	'****************************************************************************/
	
	'/* More Data Groups may be added in the future.
	' * Possible candidates include text, vector graphics, sound, etc.
	' * NOTE: Data Group constants must be powers of 2 as they are used
	' *       as bitflags when Application asks DSM to present a list of DSs.
	' */
	
	Public Const DG_CONTROL As Short = &H1s '/* data pertaining to control       */
	Public Const DG_IMAGE As Short = &H2s '/* data pertaining to raster images */
	'/* Added 1.8 */
	Public Const DG_AUDIO As Short = &H4s '/* data pertaining to audio */
	
	'/* Return Codes: DSM_Entry and DS_Entry may return any one of these values. */
	Public Const TWRC_CUSTOMBASE As Short = &H8000s
	
	Public Const TWRC_SUCCESS As Short = 0
	Public Const TWRC_FAILURE As Short = 1 '/* Application may get TW_STATUS for info on failure */
	Public Const TWRC_CHECKSTATUS As Short = 2 '/* "tried hard"; get status                  */
	Public Const TWRC_CANCEL As Short = 3
	Public Const TWRC_DSEVENT As Short = 4
	Public Const TWRC_NOTDSEVENT As Short = 5
	Public Const TWRC_XFERDONE As Short = 6
	Public Const TWRC_ENDOFLIST As Short = 7 '/* After MSG_GETNEXT if nothing left         */
	Public Const TWRC_INFONOTSUPPORTED As Short = 8
	Public Const TWRC_DATANOTAVAILABLE As Short = 9
	
	
	'/****************************************************************************
	'* Generic Constants                                                        *
	'****************************************************************************/
	
	Public Const TWON_ARRAY As Short = 3 '/* indicates TW_ARRAY container       */
	Public Const TWON_ENUMERATION As Short = 4 '/* indicates TW_ENUMERATION container */
	Public Const TWON_ONEVALUE As Short = 5 '/* indicates TW_ONEVALUE container    */
	Public Const TWON_RANGE As Short = 6 '/* indicates TW_RANGE container       */
	
	Public Const TWON_ICONID As Short = 962 '/* res Id of icon used in USERSELECT lbox */
	Public Const TWON_DSMID As Short = 461 '/* res Id of the DSM version num resource */
	Public Const TWON_DSMCODEID As Short = 63 '/* res Id of the Mac SM Code resource     */
	
	Public Const TWON_DONTCARE8 As Short = &HFFs
	Public Const TWON_DONTCARE16 As Short = &HFFFFs
	Public Const TWON_DONTCARE32 As Integer = &HFFFFFFFF
	
	
	'/****************************************************************************
	'* Capabilities                                                             *
	'****************************************************************************/
	
	Public Const CAP_CUSTOMBASE As Short = &H8000s '/*Base of custom capabilities */
	
	'/*all data sources are REQUIRED to support these caps */
	Public Const CAP_XFERCOUNT As Short = &H1s
	
	'/*image data sources are REQUIRED to support these caps */
	Public Const ICAP_COMPRESSION As Short = &H100s
	Public Const ICAP_PIXELTYPE As Short = &H101s
	Public Const ICAP_UNITS As Short = &H102s '/*default is TWUN_INCHES */
	Public Const ICAP_XFERMECH As Short = &H103s
	
	'/*all data sources MAY support these caps */
	Public Const CAP_AUTHOR As Short = &H1000s
	Public Const CAP_CAPTION As Short = &H1001s
	Public Const CAP_FEEDERENABLED As Short = &H1002s
	Public Const CAP_FEEDERLOADED As Short = &H1003s
	Public Const CAP_TIMEDATE As Short = &H1004s
	Public Const CAP_SUPPORTEDCAPS As Short = &H1005s
	Public Const CAP_EXTENDEDCAPS As Short = &H1006s
	Public Const CAP_AUTOFEED As Short = &H1007s
	Public Const CAP_CLEARPAGE As Short = &H1008s
	Public Const CAP_FEEDPAGE As Short = &H1009s
	Public Const CAP_REWINDPAGE As Short = &H100As
	Public Const CAP_INDICATORS As Short = &H100Bs '/*Added 1.1 */
	Public Const CAP_SUPPORTEDCAPSEXT As Short = &H100Cs '/*Added 1.6 */
	Public Const CAP_PAPERDETECTABLE As Short = &H100Ds '/*Added 1.6 */
	Public Const CAP_UICONTROLLABLE As Short = &H100Es '/*Added 1.6 */
	Public Const CAP_DEVICEONLINE As Short = &H100Fs '/*Added 1.6 */
	Public Const CAP_AUTOSCAN As Short = &H1010s '/*Added 1.6 */
	Public Const CAP_THUMBNAILSENABLED As Short = &H1011s '/*Added 1.7 */
	Public Const CAP_DUPLEX As Short = &H1012s '/*Added 1.7 */
	Public Const CAP_DUPLEXENABLED As Short = &H1013s '/*Added 1.7 */
	Public Const CAP_ENABLEDSUIONLY As Short = &H1014s '/*Added 1.7 */
	Public Const CAP_CUSTOMDSDATA As Short = &H1015s '/*Added 1.7 */
	Public Const CAP_ENDORSER As Short = &H1016s '/*Added 1.7 */
	Public Const CAP_JOBCONTROL As Short = &H1017s '/*Added 1.7 */
	Public Const CAP_ALARMS As Short = &H1018s '/*Added 1.8 */
	Public Const CAP_ALARMVOLUME As Short = &H1019s '/*Added 1.8 */
	Public Const CAP_AUTOMATICCAPTURE As Short = &H101As '/*Added 1.8 */
	Public Const CAP_TIMEBEFOREFIRSTCAPTURE As Short = &H101Bs '/*Added 1.8 */
	Public Const CAP_TIMEBETWEENCAPTURES As Short = &H101Cs '/*Added 1.8 */
	Public Const CAP_CLEARBUFFERS As Short = &H101Ds '/*Added 1.8 */
	Public Const CAP_MAXBATCHBUFFERS As Short = &H101Es '/*Added 1.8 */
	Public Const CAP_DEVICETIMEDATE As Short = &H101Fs '/*Added 1.8 */
	Public Const CAP_POWERSUPPLY As Short = &H1020s '/*Added 1.8 */
	Public Const CAP_CAMERAPREVIEWUI As Short = &H1021s '/*Added 1.8 */
	Public Const CAP_DEVICEEVENT As Short = &H1022s '/*Added 1.8 */
	Public Const CAP_SERIALNUMBER As Short = &H1024s '/*Added 1.8 */
	Public Const CAP_PRINTER As Short = &H1026s '/*Added 1.8 */
	Public Const CAP_PRINTERENABLED As Short = &H1027s '/*Added 1.8 */
	Public Const CAP_PRINTERINDEX As Short = &H1028s '/*Added 1.8 */
	Public Const CAP_PRINTERMODE As Short = &H1029s '/*Added 1.8 */
	Public Const CAP_PRINTERSTRING As Short = &H102As '/*Added 1.8 */
	Public Const CAP_PRINTERSUFFIX As Short = &H102Bs '/*Added 1.8 */
	Public Const CAP_LANGUAGE As Short = &H102Cs '/*Added 1.8 */
	Public Const CAP_FEEDERALIGNMENT As Short = &H102Ds '/*Added 1.8 */
	Public Const CAP_FEEDERORDER As Short = &H102Es '/*Added 1.8 */
	Public Const CAP_REACQUIREALLOWED As Short = &H1030s '/*Added 1.8 */
	Public Const CAP_BATTERYMINUTES As Short = &H1032s '/*Added 1.8 */
	Public Const CAP_BATTERYPERCENTAGE As Short = &H1033s '/*Added 1.8 */
	
	'/*image data sources MAY support these caps */
	Public Const ICAP_AUTOBRIGHT As Short = &H1100s
	Public Const ICAP_BRIGHTNESS As Short = &H1101s
	Public Const ICAP_CONTRAST As Short = &H1103s
	Public Const ICAP_CUSTHALFTONE As Short = &H1104s
	Public Const ICAP_EXPOSURETIME As Short = &H1105s
	Public Const ICAP_FILTER As Short = &H1106s
	Public Const ICAP_FLASHUSED As Short = &H1107s
	Public Const ICAP_GAMMA As Short = &H1108s
	Public Const ICAP_HALFTONES As Short = &H1109s
	Public Const ICAP_HIGHLIGHT As Short = &H110As
	Public Const ICAP_IMAGEFILEFORMAT As Short = &H110Cs
	Public Const ICAP_LAMPSTATE As Short = &H110Ds
	Public Const ICAP_LIGHTSOURCE As Short = &H110Es
	Public Const ICAP_ORIENTATION As Short = &H1110s
	Public Const ICAP_PHYSICALWIDTH As Short = &H1111s
	Public Const ICAP_PHYSICALHEIGHT As Short = &H1112s
	Public Const ICAP_SHADOW As Short = &H1113s
	Public Const ICAP_FRAMES As Short = &H1114s
	Public Const ICAP_XNATIVERESOLUTION As Short = &H1116s
	Public Const ICAP_YNATIVERESOLUTION As Short = &H1117s
	Public Const ICAP_XRESOLUTION As Short = &H1118s
	Public Const ICAP_YRESOLUTION As Short = &H1119s
	Public Const ICAP_MAXFRAMES As Short = &H111As
	Public Const ICAP_TILES As Short = &H111Bs
	Public Const ICAP_BITORDER As Short = &H111Cs
	Public Const ICAP_CCITTKFACTOR As Short = &H111Ds
	Public Const ICAP_LIGHTPATH As Short = &H111Es
	Public Const ICAP_PIXELFLAVOR As Short = &H111Fs
	Public Const ICAP_PLANARCHUNKY As Short = &H1120s
	Public Const ICAP_ROTATION As Short = &H1121s
	Public Const ICAP_SUPPORTEDSIZES As Short = &H1122s
	Public Const ICAP_THRESHOLD As Short = &H1123s
	Public Const ICAP_XSCALING As Short = &H1124s
	Public Const ICAP_YSCALING As Short = &H1125s
	Public Const ICAP_BITORDERCODES As Short = &H1126s
	Public Const ICAP_PIXELFLAVORCODES As Short = &H1127s
	Public Const ICAP_JPEGPIXELTYPE As Short = &H1128s
	Public Const ICAP_TIMEFILL As Short = &H112As
	Public Const ICAP_BITDEPTH As Short = &H112Bs
	Public Const ICAP_BITDEPTHREDUCTION As Short = &H112Cs '/*Added 1.5 */
	Public Const ICAP_UNDEFINEDIMAGESIZE As Short = &H112Ds '/*Added 1.6 */
	Public Const ICAP_IMAGEDATASET As Short = &H112Es '/*Added 1.7 */
	Public Const ICAP_EXTIMAGEINFO As Short = &H112Fs '/*Added 1.7 */
	Public Const ICAP_MINIMUMHEIGHT As Short = &H1130s '/*Added 1.7 */
	Public Const ICAP_MINIMUMWIDTH As Short = &H1131s '/*Added 1.7 */
	Public Const ICAP_FLIPROTATION As Short = &H1136s '/*Added 1.8 */
	Public Const ICAP_BARCODEDETECTIONENABLED As Short = &H1137s '/*Added 1.8 */
	Public Const ICAP_SUPPORTEDBARCODETYPES As Short = &H1138s '/*Added 1.8 */
	Public Const ICAP_BARCODEMAXSEARCHPRIORITIES As Short = &H1139s '/*Added 1.8 */
	Public Const ICAP_BARCODESEARCHPRIORITIES As Short = &H113As '/*Added 1.8 */
	Public Const ICAP_BARCODESEARCHMODE As Short = &H113Bs '/* Added 1.8 */
	Public Const ICAP_BARCODEMAXRETRIES As Short = &H113Cs '/* Added 1.8 */
	Public Const ICAP_BARCODETIMEOUT As Short = &H113Ds '/*Added 1.8 */
	Public Const ICAP_ZOOMFACTOR As Short = &H113Es '/*Added 1.8 */
	Public Const ICAP_PATCHCODEDETECTIONENABLED As Short = &H113Fs '/*Added 1.8 */
	Public Const ICAP_SUPPORTEDPATCHCODETYPES As Short = &H1140s '/*Added 1.8 */
	Public Const ICAP_PATCHCODEMAXSEARCHPRIORITIES As Short = &H1141s '/*Added 1.8 */
	Public Const ICAP_PATCHCODESEARCHPRIORITIES As Short = &H1142s '/*Added 1.8 */
	Public Const ICAP_PATCHCODESEARCHMODE As Short = &H1143s '/*Added 1.8 */
	Public Const ICAP_PATCHCODEMAXRETRIES As Short = &H1144s '/*Added 1.8 */
	Public Const ICAP_PATCHCODETIMEOUT As Short = &H1145s '/*Added 1.8 */
	Public Const ICAP_FLASHUSED2 As Short = &H1146s '/*Added 1.8 */
	Public Const ICAP_IMAGEFILTER As Short = &H1147s '/*Added 1.8 */
	Public Const ICAP_NOISEFILTER As Short = &H1148s '/*Added 1.8 */
	Public Const ICAP_OVERSCAN As Short = &H1149s '/*Added 1.8 */
	Public Const ICAP_AUTOMATICBORDERDETECTION As Short = &H1150s '/*Added 1.8 */
	Public Const ICAP_AUTOMATICDESKEW As Short = &H1151s '/*Added 1.8 */
	Public Const ICAP_AUTOMATICROTATE As Short = &H1152s '/*Added 1.8 */
	Public Const ICAP_JPEGQUALITY As Short = &H1153s '/*Added 1.9 */
	
	
	'/* ICAP_UNITS values (UN_ means UNits) */
	Public Const TWUN_INCHES As Short = 0
	Public Const TWUN_CENTIMETERS As Short = 1
	Public Const TWUN_PICAS As Short = 2
	Public Const TWUN_POINTS As Short = 3
	Public Const TWUN_TWIPS As Short = 4
	Public Const TWUN_PIXELS As Short = 5
	
	
	Structure TW_STR1024
		'UPGRADE_NOTE: str  str_Renamed ɃAbvO[h܂B ڍׂɂẴ͎NNbNĂ : 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="vbup1061"'
		<VBFixedString(1025),System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValTStr,SizeConst:=1025)> Public str_Renamed As String
    End Structure

    Public Structure TW_UINT32
        Dim l As Short
        Dim u As Short
    End Structure

    Structure TW_FIX32
        Dim Whole As Short '/* maintains the sign */
        Dim Frac As Short
    End Structure


    '/* No DAT needed.  Describes version of software currently running. */
    Structure TW_VERSION
        Dim MajorNum As Short ' /* Major revision number of the software. */
        Dim MinorNum As Short ' /* Incremental revision number of the software. */
        Dim Language As Short ' /* e.g. TWLG_SWISSFRENCH */
        Dim Country As Short ' /* e.g. TWCY_SWITZERLAND */
        <VBFixedString(34), System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst:=34)> Public Info As String ' /* e.g. "1.0b3 Beta release" */
    End Structure

    '/* DAT_IDENTITY. Identifies the program/library/code resource. */
    Public Structure TW_IDENTITY
        Dim Id As TW_UINT32 '/* Unique number.  In Windows, application hWnd */
        Dim Version As TW_VERSION '/* Identifies the piece of code              */
        Dim ProtocolMajor As Short '/* Application and DS must set to TWON_PROTOCOLMAJOR */
        Dim ProtocolMinor As Short '/* Application and DS must set to TWON_PROTOCOLMINOR */
        Dim SupportedGroups As TW_UINT32 '/* Bit field OR combination of DG_ constants */
        <VBFixedString(34), System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst:=34)> Public Manufacturer As String '/* Manufacturer name, e.g. "Hewlett-Packard" */
        <VBFixedString(34), System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst:=34)> Public ProductFamily As String '/* Product family name, e.g. "ScanJet"       */
        <VBFixedString(34), System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst:=34)> Public ProductName As String '/* Product name, e.g. "ScanJet Plus"         */
    End Structure


    '/* DAT_IMAGEINFO. Application gets detailed image info from DS with this. */
    Structure TW_IMAGEINFO
        Dim XResolution As TW_FIX32 '/* Resolution in the horizontal             */
        Dim YResolution As TW_FIX32 '/* Resolution in the vertical               */
        Dim ImageWidth As Integer '/* Columns in the image, -1 if unknown by DS*/
        Dim ImageLength As Integer '/* Rows in the image, -1 if unknown by DS   */
        Dim SamplesPerPixel As Short '/* Number of samples per pixel, 3 for RGB   */
        <VBFixedArray(7)> Dim BitsPerSample() As Short '/* Number of bits for each sample           */
        Dim BitsPerPixel As Short '/* Number of bits for each padded pixel     */
        Dim Planar As Short '/* True if Planar, False if chunky          */
        Dim PixelType As Short '/* How to interp data; photo interp (TWPT_) */
        Dim Compression As Short '/* How the data is compressed (TWCP_xxxx)   */

        Public Sub Initialize()
            ReDim BitsPerSample(7)
        End Sub
    End Structure

    '/* No DAT.  Defines a frame rectangle in ICAP_UNITS coordinates. */
    Structure TW_FRAME
        Dim Left As TW_FIX32
        Dim Top As TW_FIX32
        Dim Right As TW_FIX32
        Dim Bottom As TW_FIX32
    End Structure

    '/* DAT_IMAGELAYOUT. Provides image layout information in current units. */
    Public Structure TW_IMAGELAYOUT
        Dim Frame As TW_FRAME          '/* Frame coords within larger document */
        Dim DocumentNumber As TW_UINT32
        Dim PageNumber As TW_UINT32      '/* Reset when you go to next document  */
        Dim FrameNumber As TW_UINT32      '/* Reset when you go to next page      */
    End Structure
End Module